#!/usr/dist/pkgs/perl,v5.003/5bin.sun4/perl
require 5.003;

BEGIN {
    my($workdir) = ($0 =~ /(.*)\/.*/);
    unshift(@INC,"$workdir/../lib");
}
my($workdir) = ($0 =~ /(.*)\/.*/);
use Socket;
use BER;
use SNMP_Session;
use SNMP_MIB;
use SNMP_Simple;
use strict;

sub parse_query;
sub query_to_html_response;
sub write_query_form;

# Format a number
sub fmi {
  my($number) = $_[0];
  my(@short);
  @short = ("Bits/s","kbitss/s","Mbits/s","Gbits/s");
  my $digits=length("".$number);
  my $divm=0;
  while ($digits-$divm*3 > 4) { $divm++; }
  my $divnum = $number/10**($divm*3);
  return sprintf("%1.1f %s",$divnum,$short[$divm]);
}

my(%status_d) = ('1'=>'Up',
		 '2'=>'Down',
		 '3'=>'Testing',
		 '4'=>'Dormant',
		 '5'=>'notPresent');
my(%ifType_d)=('1'=>'Other',
	       '2'=>'regular1822',
	       '3'=>'hdh1822',
	       '4'=>'ddnX25',
	       '5'=>'rfc877x25',
	       '6'=>'ethernetCsmacd',
	       '7'=>'iso88023Csmacd',
	       '8'=>'iso88024TokenBus',
	       '9'=>'iso88025TokenRing',
	       '10'=>'iso88026Man',
	       '11'=>'starLan',
	       '12'=>'proteon10Mbit',
	       '13'=>'proteon80Mbit',
	       '14'=>'hyperchannel',
	       '15'=>'fddi',
	       '16'=>'lapb',
	       '17'=>'sdlc',
	       '18'=>'ds1',
	       '19'=>'e1',
	       '20'=>'basicISDN',
	       '21'=>'primaryISDN',
	       '22'=>'propPointToPointSerial',
	       '23'=>'ppp',
	       '24'=>'softwareLoopback',
	       '25'=>'eon',
	       '26'=>'ethernet-3Mbit',
	       '27'=>'nsip',
	       '28'=>'slip',
	       '29'=>'ultra',
	       '30'=>'ds3',
	       '31'=>'sip',
	       '32'=>'frame-relay',
	       '33'=>'rs232',
	       '34'=>'para',
	       '35'=>'arcnet',
	       '36'=>'arcnetPlus',
	       '37'=>'atm',
	       '38'=>'miox25',
	       '39'=>'sonet',
	       '40'=>'x25ple',
	       '41'=>'iso88022llc',
	       '42'=>'localTalk',
	       '43'=>'smdsDxi',
	       '44'=>'frameRelayService',
	       '45'=>'v35',
	       '46'=>'hssi',
	       '47'=>'hippi',
	       '48'=>'modem',
	       '49'=>'aal5',
	       '50'=>'sonetPath',
	       '51'=>'sonetVT',
	       '52'=>'smdsIcip',
	       '53'=>'propVirtual',
	       '54'=>'propMultiplexor',
	       '55'=>'100BaseVG',
	       '56'=>'fibre-channel',
	       '59'=>'aflane8023',
	       '60'=>'aflane8025',
	       '80'=>'atmLogical',
	       );


my $query_string = $ARGV[0];

if (!defined ($query_string) || !$query_string) {
} else {
    my (%query) = parse_query ();
    SNMP_MIB::loadoids("$workdir/../mibs", "RFC1213", "CISCO-STACK", "CISCO-PRODUCTS",
	     "OLD-CISCO-INTERFACES", "sun-snmp", "RFC1315");
    query_to_html_response ($query{'hostname'}, $query{'community'}, 
			    $query{'interface'});
}
1;

sub parse_query
{
    my (%query) = (());
    if ($query_string =~ /\d:.*/) {
	($query{'interface'}, $query_string) = split(/:/, $query_string, 2);
    }
    if ($query_string =~ /.*@.*/) {
	($query{'community'}, $query{'hostname'}) = split('@', $query_string, 3);
    } else {
	$query{'hostname'} = $query_string;
	$query{'community'} = 'public';
    }
    return %query;
}

sub query_to_html_response
{
    my($hostname, $community, $interface) = @_;
    my($session) = SNMP_Simple->open($hostname, $community);
    my($sysDescr, $sysContact, $sysLocation, $sysName, $sysUpTime, 
       $sysObjectID, $interfaceNumber) 
	=  $session->snmpget('sysDescr', 'sysContact',
		   'sysLocation', 'sysName', 'sysUpTime',
		   'sysObjectID', 'ifNumber');
    $sysDescr =~ y/\r/\n/;
    my($sysObjectName) = SNMP_MIB::oid2name($sysObjectID);
    my($cisco_router_sysid) = '1\.3\.6\.1\.4\.1\.9\.1';
    my($cisco_switch_sysid) = '1\.3\.6\.1\.4\.1\.9\.5';
    my($ciscorouter) = ($sysObjectID =~ /^$cisco_router_sysid/);
    my($ciscoswitch) = ($sysObjectID =~ /^$cisco_switch_sysid/);

    print <<EOF;
Name\t\t$sysName
Description\t$sysDescr
Contact\t$sysContact
Location\t$sysLocation
Uptime\t\t$sysUpTime
ObjectID\t$sysObjectID: $sysObjectName
EOF
    if ($interface ne "") {
	my($circuit);
	if ($interface =~ /^\d+\.FR\d+/) {
	    ($interface, $circuit) = ($interface =~ /^(\d+)\.FR(\d+)/);
	}
	my($ipaddr, $iphost, $index);
	my @ipadent = $session->snmpgettable('ipAdEntAddr');
	my @ipadentif = $session->snmpgettable('ipAdEntIfIndex');
	while (scalar @ipadentif){
	    $index = shift @ipadentif;
	    my($tmp) = shift @ipadent;
	    if ($index == $interface) {
		$ipaddr = $tmp;
		$iphost = 
		    gethostbyaddr(pack('C4',split(/\./,$ipaddr)), AF_INET);
		if ($iphost eq ''){
		    $iphost = $ipaddr;
		}
	    }
	}
	my($ifdesc, $iftype, $ifSpeed, $ifAdmin, $ifOper) = 
	    $session->snmpget("ifDescr.$interface", 
		    "ifType.$interface", "ifSpeed.$interface",
		    "ifAdminStatus.$interface", "ifOperStatus.$interface");
	my $speed_str = &fmi(int($ifSpeed));
	my($cdesc);
#	if ($ciscorouter) {
	    my($cdesc, $cinbits, $coutbits) 
		= $session->snmpget("locIfDescr.$interface",
				    "locIfInBitsSec.$interface",
				    "locIfOutBitsSec.$interface");
	    print "Interface \#$interface\t$ifdesc: $cdesc\n";
	    print "Interface \#$interface in bitrate\t\t$cinbits\n";
	    print "Interface \#$interface out bitrate\t$coutbits\n";
#	} else {
#	    print "Interface \#$interface\t$ifdesc\n";	
#	}
	if ($ipaddr ne "") {
	    print "IP Address\t$ipaddr ($iphost)\n";
	}

	print <<EOF;
Interface Type\t$ifType_d{$iftype}
Interface Speed\t$speed_str
Interface Status\tAdmin: $status_d{$ifAdmin} Oper: $status_d{$ifOper}
EOF
        if (defined($circuit)) {
	    my($frstatus) = snmpget($hostname, $community, "frCircuitState.$interface.$circuit");
	    if ($frstatus == 1) {
		$frstatus = 'invalid';
	    } elsif ($frstatus == 2) {
		$frstatus = 'active';
	    } elsif ($frstatus == 3) {
		$frstatus = 'inactive';
	    }
	    print "Circuit Status\tCircuit ID $circuit $frstatus\n";
        }
     }
	
    1;
}
